/*
  bounce for ikkeiplexing shield by ikkei
  http://blog.goo.ne.jp/jh3kxm
  
  SW1: calibration
  SW4: reverse
  
  History:
        2010-12-11 - V1.0 bounce by ikkei
        
*/

#include <MsTimer2.h>
#include <Charlieplexing.h>
#include <EEPROM.h>


#define AX 4
#define AY 5
#define AVE 8 // averaging number of times
#define REF1 3  // Reflectance is REF1/REF2
#define REF2 4
#define MAG 1024  // Magnification
#define SENS 1  // Sensitivity
#define FRAME 1

const uint8_t X_NUM = 16;
const uint8_t Y_NUM = 8;
const uint8_t X_MAX = 15;
const uint8_t Y_MAX = 7;

/** switch bit patteren */
const uint8_t SW1 = 0x80;
const uint8_t SW2 = 0x40;
const uint8_t SW3 = 0x20;
const uint8_t SW4 = 0x10;
const uint8_t SW5 = 0x08;
const uint8_t SW6 = 0x04;
const uint8_t SW7 = 0x02;
const uint8_t SW8 = 0x01;

int i;
int x = 1;  // display next position
int y = 1;
int x0 = 1;  // display last position
int y0 = 1;
int acc_xin[AVE];  // sensor signal averaging array
int acc_yin[AVE];
byte spt = 0;  // averaging pointer
int acc_x;  // acceleration
int acc_y;
int vel_x;  // velocity
int vel_y;
int pos_x;  // position
int pos_y;
int offset_x;
int offset_y;
int pol_x; // Polarity x
int pol_y; // Polarity y

void calc_acc( void ){
  // averaging acceleration sensor signal
  acc_xin[ spt ] = analogRead( AX );
  acc_yin[ spt ] = analogRead( AY );
  spt++;
  if ( spt >= AVE ){
    spt = 0;
  }
  acc_x = 0;
  acc_y = 0;
  for ( i=0; i < AVE; i++){
    acc_x += ( acc_xin[ i ] - offset_x ) * pol_x;
    acc_y += ( acc_yin[ i ] - offset_y ) * pol_y;
  }
  acc_x /= AVE;
  acc_y /= AVE;
  
  // integrating acceleration makes velocity
  vel_x += acc_x * SENS;
  vel_y += acc_y * SENS;
  
  // integrating velocity makes position
  pos_x += vel_x;
  pos_y += vel_y;
  
  // reflection ( reverse velocity )
  // 0 < x < X_MAX
  if ( pos_x >= MAG * X_MAX){
    pos_x = MAG * X_MAX - 1;
    vel_x = -( vel_x * REF1 ) / REF2;
  }else if ( pos_x < MAG ){
    pos_x = MAG;
    vel_x = -( vel_x * REF1 ) / REF2;
  }
  // 0 < y < Y_MAX
  if ( pos_y >= MAG * Y_MAX){
    pos_y = MAG * Y_MAX - 1;
    vel_y = -( vel_y * REF1 ) / REF2;
  }else if ( pos_y < MAG ){
    pos_y = MAG;
    vel_y = -( vel_y * REF1 ) / REF2;
  }
  
//  x = acc_x + 8;    // test acceleration value
//  y = acc_y + 4;
//  x = vel_x / 16 + 8;  // test velocity value
//  y = vel_y / 16 + 4;
  x = pos_x / MAG;
  y = pos_y / MAG;

  // display limitter
  if ( x > (X_MAX - 1)) {
    x = X_MAX - 1;
  }else if ( x < 1 ){
    x = 1;
  }
  if ( y > (Y_MAX - 1)) {
    y = Y_MAX - 1;
  }else if (y < 1){
    y = 1;
  }

  // turn off the last position
  iLedSign::Set(x0, y0, 0);
  iLedSign::Set(x0+1, y0, 0);
  iLedSign::Set(x0-1, y0, 0);
  iLedSign::Set(x0, y0+1, 0);
  iLedSign::Set(x0, y0-1, 0);
  
  // turn on the next position
  iLedSign::Set(x, y, 1);
  iLedSign::Set(x+1, y, 1);
  iLedSign::Set(x-1, y, 1);
  iLedSign::Set(x, y+1, 1);
  iLedSign::Set(x, y-1, 1);
  
  // store last position
  x0 = x;
  y0 = y;
}

void setup(){
  iLedSign::Init();  
  offset_x = EEPROM.read(0)*256 + EEPROM.read(1);
  offset_y = EEPROM.read(2)*256 + EEPROM.read(3);
  pol_x = 1;
  pol_y = -1;
}

void loop(){
  if ((iLedSign::GetPeriod() & FRAME) != 0){
    byte keydata = iLedSign::GetSW(SW_EDGE); // get switch edge
    calc_acc();
  
    if ((keydata & SW1) !=0 ){
      offset_x = acc_xin[ 0 ];
      offset_y = acc_yin[ 0 ];
      EEPROM.write(0, offset_x / 256);
      EEPROM.write(1, offset_x % 256);
      EEPROM.write(2, offset_y / 256);
      EEPROM.write(3, offset_y % 256);
    }
    if ((keydata & SW4) !=0 ){
      pol_x *= -1;
      pol_y *= -1;
    }
  }
}


